package main

import (
	"context"
	"encoding/json"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"net/url"
	"os"
	"strconv"
	"strings"
	"sync"
	"time"

	"flag"

	"github.com/IBM/sarama"
	"github.com/gofiber/fiber/v2"

	gogoPbProto "github.com/gogo/protobuf/proto"
	pbUpstream "github.com/odysseus/odysseus-protocol/gen/proto/go/base/v1"
	qdb "github.com/questdb/go-questdb-client/v2"

	"github.com/odysseus/payment/cachedata"
	"github.com/odysseus/payment/model"
)

var (
	// Update with your Kafka broker addresses
	kafkaBrokers = []string{"localhost:9092"}

	callbackAddr = ""

	// Update with your Kafka topic name
	topic = "test_topic"
	// Set an appropriate buffer size based on your requirements
	maxBufferSize = 100

	aigcTopic = "pbaigc"
)

// Global counter to keep track of the messages
var counter int

// Internal slice to store messages received from Kafka
var messages = make([]string, 0, maxBufferSize)

// Mutex to synchronize access to the messages slice
var mutex = sync.Mutex{}

// Channel to send messages from the /producer endpoint to the Kafka producer goroutine
var producerMessages = make(chan string)

type bytesAndHeader struct {
	Bytes      []byte
	HttpHeader map[string]string
}

var producerMessagesBytes = make(chan bytesAndHeader, 1000)

//var TaskIdInstream = make(chan string, 1000)

// Channel to send messages from the Kafka consumer to the /consumer endpoint
var consumerMessages = make(chan string)

func kafkaProducerBytes() {

	// Create a new Sarama configuration for the Kafka producer.
	config := sarama.NewConfig()

	// Create a new Kafka producer using the specified configuration and broker addresses.
	producer, err := sarama.NewAsyncProducer(kafkaBrokers, config)

	if err != nil {
		panic(fmt.Sprint("Failed to start Kafka producer:", err))
	}

	// Ensure the Kafka producer is closed when the function ends (deferred execution).
	defer producer.Close()

	for message := range producerMessagesBytes {

		v := append(sarama.ByteEncoder{}, message.Bytes...)

		//sarama.ByteEncoder =
		producer.Input() <- &sarama.ProducerMessage{
			Topic: aigcTopic,
			Value: v,
		}

		// Introduce random delay between 1 to 3 seconds for message push
		//time.Sleep(time.Duration(1+rand.Intn(3)) * time.Second)
	}
}

func kafkaConsumerBytes(done chan interface{}, req, resTopic string) (chan pbUpstream.TaskContent, chan pbUpstream.TaskReceipt) {

	reqOutStream := make(chan pbUpstream.TaskContent, 1000)

	resOutStream := make(chan pbUpstream.TaskReceipt, 1000)

	go func() {

		// Create a new Sarama configuration for the Kafka producer.
		config := sarama.NewConfig()

		// Create a new Kafka consumer using the specified configuration and broker addresses.
		consumer, err := sarama.NewConsumer(kafkaBrokers, config)

		if err != nil {
			panic(fmt.Sprint("Failed to start Kafka consumer:", err))
		}

		// Ensure the Kafka consumer is closed when the function ends (deferred execution).
		defer consumer.Close()

		// Create a partition consumer for the specified topic, partition, and starting offset.
		// The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
		reqConsumer, err := consumer.ConsumePartition(req, 0, sarama.OffsetNewest)

		if err != nil {
			panic(fmt.Sprint("Failed to start partition consumer:", err))
		}

		// Ensure the partition consumer is closed when the function ends (deferred execution).
		defer reqConsumer.Close()

		for {
			select {
			case message := <-reqConsumer.Messages():

				if message == nil {
					slog.Error("kafka consumer", "topic", req, "message", "is nil")
					continue
				}

				if message.Value == nil {
					slog.Error("kafka consumer", "topic", req, "message vaule", "is nil")
					continue
				}

				pbMsg := pbUpstream.TaskContent{}

				if err := gogoPbProto.Unmarshal(message.Value, &pbMsg); err != nil {
					slog.Error("kafka consumer", "topic", req, "gogoPbProto.Unmarshal err", err.Error())
					continue
				}

				slog.Info("kafka consumer", "topic", req, "pbMsg.TaskId", pbMsg.TaskId)

				select {
				case <-done:
					return
				case reqOutStream <- pbMsg:
				}
			}
		}

	}()

	go func() {
		// Create a new Sarama configuration for the Kafka producer.
		config := sarama.NewConfig()

		// Create a new Kafka consumer using the specified configuration and broker addresses.
		consumer, err := sarama.NewConsumer(kafkaBrokers, config)
		if err != nil {
			panic(fmt.Sprint("Failed to start Kafka consumer:", err))
		}

		// Ensure the Kafka consumer is closed when the function ends (deferred execution).
		defer consumer.Close()

		// Create a partition consumer for the specified topic, partition, and starting offset.
		// // The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
		// reqConsumer, err := consumer.ConsumePartition(req, 0, sarama.OffsetNewest)
		// if err != nil {
		// 	log.Fatal("Failed to start partition consumer:", err)
		// }

		// // Ensure the partition consumer is closed when the function ends (deferred execution).
		// defer reqConsumer.Close()

		resConsumer, err := consumer.ConsumePartition(resTopic, 0, sarama.OffsetNewest)

		if err != nil {
			panic(fmt.Sprint("Failed to start partition consumer:", err))
		}

		// Ensure the partition consumer is closed when the function ends (deferred execution).
		defer resConsumer.Close()

		// Signal that the consumer goroutine is ready
		//wg.Done()

		// Infinite loop to continuously listen for messages from the partitionConsumer.Messages() channel.

		for {
			select {

			case message := <-resConsumer.Messages():

				if message == nil {
					slog.Error("kafka consumer", "topic", resTopic, "message", "is nil")
					continue
				}

				if message.Value == nil {
					slog.Error("kafka consumer", "topic", resTopic, "message vaule", "is nil")
					continue
				}

				pbMsg := pbUpstream.TaskReceipt{}

				if err := gogoPbProto.Unmarshal(message.Value, &pbMsg); err != nil {
					slog.Error("kafka consumer", "topic", resTopic, "gogoPbProto.Unmarshal err", err.Error())
					continue
				}

				slog.Info("kafka consumer", "topic", resTopic, "pbMsg.TaskId", pbMsg.TaskId)

				select {
				case <-done:
					return
				case resOutStream <- pbMsg:
				}

			case <-done:
				return
			}
		}
	}()

	return reqOutStream, resOutStream
}

func batchToQuestDb(done chan interface{}, reqStream chan pbUpstream.TaskContent, resStream chan pbUpstream.TaskReceipt, reqTableName string, resTableName, questAddr string) {

	//tableName := "tasks"

	ctx := context.TODO()
	// Connect to QuestDB running on 127.0.0.1:9009
	//addrOpt := qdb.WithAddress("192.168.1.220:9009")
	addrOpt := qdb.WithAddress(questAddr)

	sender, err := qdb.NewLineSender(ctx, addrOpt)
	if err != nil {
		panic(err)
	}
	// Make sure to close the sender on exit to release resources.
	defer sender.Close()

	for {
		select {
		case task := <-resStream:

			nanoseconds := int64(task.TaskTimestamp)
			seconds := nanoseconds / 1e9

			slog.Debug("questdb <- resStream", "task.TaskId", task.TaskId, "task.TaskUid", task.TaskUid)

			sender.Table(resTableName).
				Symbol("type", fmt.Sprintf("%d", task.TaskType)).
				Symbol("uid", task.TaskUid).
				Symbol("profit_acc", task.TaskProfitAccount).
				Symbol("worker_acc", task.TaskWorkerAccount).
				Symbol("result", task.TaskResult).
				StringColumn("id", task.TaskId).
				TimestampColumn("time", time.Unix(seconds, nanoseconds%1e9)).
				Int64Column("fee", task.TaskFee).
				Int64Column("workload", int64(task.TaskWorkload)).
				Int64Column("out_len", int64(task.TaskOutLen)).
				Int64Column("task_duration", int64(task.TaskDuration)).
				//task_execute_duration
				Int64Column("exec_duration", int64(task.TaskExecuteDuration)).
				AtNow(ctx)

			err = sender.Flush(ctx)
			if err != nil {
				slog.Error("task := <-resStream", "error", err.Error())
				//log.Fatal(err)
			}

		case task := <-reqStream:

			// sender.
			// 	Table(reqTableName).
			// 	Symbol("type", fmt.Sprintf("%d", randomType())).
			// 	Symbol("uid", fmt.Sprintf("%d", randomUID())).
			// 	StringColumn("id", uuid.New().String()).
			// 	TimestampColumn("time", time.Now()).
			// 	Int64Column("fee", int64(randomFeeAndWorkload())).
			// 	Int64Column("in_len", int64(randomInput())).
			// 	AtNow(ctx)

			_ = task
			taskFeeAsInt, err := strconv.Atoi(task.TaskFee)
			if err != nil {
				slog.Error("task.TaskFee string to int", "error", err)
				continue
			}

			slog.Debug("questdb <- reqStream", "task.TaskId", task.TaskId, "task.TaskUid", task.TaskUid)

			nanoseconds := int64(task.TaskTimestamp)
			seconds := nanoseconds / 1e9

			// 使用Unix函数转换为time.Time
			// t := time.Unix(seconds, nanoseconds%1e9)
			// time.Unix()

			sender.
				Table(reqTableName).
				Symbol("type", fmt.Sprintf("%d", task.TaskType)).
				Symbol("uid", task.TaskUid).
				StringColumn("id", task.TaskId).
				TimestampColumn("time", time.Unix(seconds, nanoseconds%1e9)).
				Int64Column("fee", int64(taskFeeAsInt)).
				Int64Column("in_len", int64(task.TaskInLen)).
				AtNow(ctx)

			err = sender.Flush(ctx)
			if err != nil {
				slog.Error("task := <-reqStream", "error", err.Error())
				//fmt.Println("task := <-reqStream sender.Flush", err.Error())
				//log.Fatal(err)
			}

		case <-done:
			return

		}

	}
}

type ResponseJson struct {
	TaskId     string
	TaskResult []byte
	TaskUid    string
	TaskFee    string
	IsSuccess  bool
	TaskError  string
}

func newCache(redisAddr, redisPass, mysqlIP, dbName, user, passwd string, port int) *cachedata.CacheData {

	_cache := cachedata.NewCacheData(context.Background(), cachedata.RedisConnParam{
		//Addr:     "192.168.1.10:6379",
		Addr:     redisAddr,
		Password: redisPass,
		DbIndex:  0,
	}, model.DbConfig{
		//Host:   "192.168.1.211",
		Host:   mysqlIP,
		Port:   port,
		DbName: dbName,
		Passwd: passwd,
		User:   user,
		// DbName: "liuxuzhong",
		// Passwd: "12345678",
		// User:   "root",
	})
	return _cache
}

var cache *cachedata.CacheData

var createTaskTableSql string = `CREATE TABLE IF NOT EXISTS 
'tasks' ( 
	id UUID, 
	time TIMESTAMP, 
	type symbol CAPACITY 128 INDEX CAPACITY 1048576, 
	uid symbol CAPACITY 1024 INDEX CAPACITY 1048576, 
	fee INT, 
	in_len INT 
   ) timestamp (time) PARTITION BY DAY WAL;`

var createBillsTableSql string = `CREATE TABLE IF NOT EXISTS 
'bills' ( 
	id UUID, 
	time TIMESTAMP, 
	type symbol CAPACITY 128 CACHE INDEX CAPACITY 8192, 
	uid symbol CAPACITY 128 CACHE INDEX CAPACITY 8192,
	fee INT, 
	out_len INT,
	workload INT, 
	task_duration INT,
	exec_duration INT,
	profit_acc symbol CAPACITY 128 CACHE INDEX CAPACITY 8192,
	worker_acc symbol CAPACITY 128 CACHE INDEX CAPACITY 8192, 
	result symbol CAPACITY 128 CACHE INDEX CAPACITY 8192
  ) timestamp (time) PARTITION BY DAY WAL;`

func createTable(ipAddr, tableSql string) {
	//"http://localhost:9000"
	u, err := url.Parse("http://" + ipAddr + ":9000")
	checkErr(err)

	u.Path += "exec"
	params := url.Values{}

	params.Add("query", tableSql)

	u.RawQuery = params.Encode()
	url := fmt.Sprintf("%v", u)

	res, err := http.Get(url)
	checkErr(err)

	defer res.Body.Close()

	body, err := io.ReadAll(res.Body)
	checkErr(err)

	slog.Info("createTable", "response body", body)

}

func checkErr(err error) {
	if err != nil {
		panic(err)
	}
}

var taskJsonStr = `{"id":10,"desc":"测试新增","price":1000,"complexity":1,"hardware_require":"{ disk_size: 100, memory_size: 100 }","image_id":"13221312","image_url":"dasdasdasd","cmd":"{\"image_name\":\"demianhjw/aigic:0129\",\"docker_cmd\":{\"container_port\":\"5001\"},\"api_url\":\"http://127.0.0.1:%d/aigic\"}","workload":100,"api_path":"/demianhjw/aigic/0129","image_name":"demianhjw/aigic:0129","sign_url":"dsdsdsds","username":"sdsds","password":"sdsdsd","created_time":"2024-02-02T03:13:33+08:00","updated_time":"2024-02-02T03:13:33+08:00","deleted":0}`
var withBillDb bool

func main() {

	slog.SetDefault(slog.New(slog.NewTextHandler(os.Stderr, &slog.HandlerOptions{
		Level: slog.LevelDebug,
		//AddSource: true,
	})))

	var questAddr, kafkaBroker, callbackAddrP, listenIpPort, aigcProduceTopic, aigcConsumerTopic string

	var redisAddr, redisPassWd, mysqlAddr string
	var mysqlPort int

	var mysqlDbName, mysqlUser, mysqlPassWd string

	//var redisPassWd string

	//flag.StringVar(&questAddr, "questAddr", "192.168.1.10:9009", "questDbAddr")

	flag.StringVar(&questAddr, "questAddr", "localhost:9009", "questDbAddr")
	flag.StringVar(&kafkaBroker, "kafkaBroker", "localhost:9092", "single kafka broker")
	flag.StringVar(&redisAddr, "redisAddr", "localhost:6379", "Redis Addr")
	flag.StringVar(&redisPassWd, "redisPassWd", "", "Redis password")
	flag.StringVar(&mysqlAddr, "mysqlAddr", "localhost", "onely ip address")
	flag.IntVar(&mysqlPort, "mysqlPort", 3306, "onely port")
	flag.StringVar(&mysqlDbName, "mysqlDbName", "liuxuzhong", "mysql db name")
	flag.StringVar(&mysqlUser, "mysqlUser", "liuxuzhong", "connect to mysql user")
	flag.StringVar(&mysqlPassWd, "mysqlPassWd", "12345678", "mysql user password")

	flag.StringVar(&callbackAddrP, "callbackIpAddr", "localhost:6001", "ip:port")
	flag.StringVar(&listenIpPort, "listenIpPort", "0.0.0.0:6001", "api listen on ip:port")
	flag.StringVar(&aigcProduceTopic, "aigcProduceTopic", "pbaigc", "produce topic, default value is: pbaigc")
	flag.StringVar(&aigcConsumerTopic, "aigcConsumerTopic", "taskreceipt", "consumer topic, default value is: taskreceipt")

	flag.BoolVar(&withBillDb, "withbilldb", true, "enable with bill db, or simulate")

	flag.Parse()

	slog.Warn("start param", "quest", slog.GroupValue(slog.String("Addr", questAddr)))
	slog.Warn("start param", "kafka", slog.GroupValue(slog.String("Addr", kafkaBroker)), slog.Group("topic", "aigcProduceTopic", aigcProduceTopic, "aigcConsumerTopic", aigcConsumerTopic))
	slog.Warn("start param", "callback", slog.GroupValue(slog.String("Addr", callbackAddrP)))

	slog.Warn("start param", "listenIpPort", slog.GroupValue(slog.String("Addr", listenIpPort)))
	slog.Warn("start param", "redis", slog.GroupValue(slog.String("Addr", redisAddr), slog.String("PassWd", redisPassWd)))
	slog.Warn("start param", "mysql", slog.GroupValue(slog.String("Addr", mysqlAddr),
		slog.Int("port", mysqlPort), slog.String("DbName", mysqlDbName), slog.String("User", mysqlUser), slog.String("PassWd", mysqlPassWd))) //, slog.GroupValue(slog.Int("port",mysqlPort),slog.String("mysqlDbName",mysqlDbName),slog.GroupValue(slog.String("mysqlUser",mysqlUser)))))

	slog.Warn("start param", "!!!!!!!!!!!!!!!!!!!!**********With Bill DB**********!!!!!!!!!!!!!!!!!!!!", withBillDb)
	slog.Warn("start param", "!!!!!!!!!!!!!!!!!!!!**********With Bill DB**********!!!!!!!!!!!!!!!!!!!!", withBillDb)
	slog.Warn("start param", "!!!!!!!!!!!!!!!!!!!!**********With Bill DB**********!!!!!!!!!!!!!!!!!!!!", withBillDb)

	kafkaBrokers = []string{kafkaBroker}
	callbackAddr = callbackAddrP

	idx := strings.Index(questAddr, ":")

	if idx == -1 {
		panic("quest addr parameter,please use the format: 1.1.1.1:9009")
	}

	createTable(questAddr[:idx], createTaskTableSql)
	createTable(questAddr[:idx], createBillsTableSql)

	// DbName: "liuxuzhong",
	// Passwd: "12345678",
	// User:   "root",

	cache = newCache(redisAddr, redisPassWd, mysqlAddr, mysqlDbName, mysqlUser, mysqlPassWd, mysqlPort)

	// Create a WaitGroup to synchronize goroutines.
	wg := &sync.WaitGroup{}
	// Add 1 to the WaitGroup, indicating one goroutine to wait for.
	wg.Add(1)

	// Launch the Kafka producer goroutine in the background.
	//go kafkaProducer()

	go kafkaProducerBytes()

	done := make(chan interface{})

	reqToQuestDb, resToQuestDb := kafkaConsumerBytes(done, aigcProduceTopic, aigcConsumerTopic)

	go batchToQuestDb(done, reqToQuestDb, resToQuestDb, "tasks", "bills", questAddr)

	go recordUUID()

	// Create a new instance of the Fiber web framework.
	app := fiber.New()

	// cfg := swagger.Config{
	// 	BasePath: "/",
	// 	FilePath: "./docs/swagger.json",
	// 	Path:     "swagger",
	// 	Title:    "Swagger API Docs",
	// }

	// app.Use(swagger.New(cfg))

	apiGroup := app.Group("/api")
	jwtGroup := app.Group("/jwt")
	callbackGroup := app.Group("/callback")

	apiGroupV1 := apiGroup.Group("/v1")
	apiGroupV1.Post("/*", ApiAndJWT)

	jwtGroupV1 := jwtGroup.Group("/v1")
	jwtGroupV1.Post("/*", ApiAndJWT)

	callbackGroupV1 := callbackGroup.Group("/v1")
	callbackGroupV1.Post("/", func(c *fiber.Ctx) error {

		slog.Debug("callback", "path", c.Route().Path)

		//fmt.Println("c.Route().Path", c.Route().Path)

		body := c.Body()

		var resbody pbUpstream.TaskResponse

		if err := gogoPbProto.Unmarshal(body, &resbody); err != nil {
			return c.SendString(fmt.Sprintf("callback Unmarshal error %v", err.Error()))
		}

		slog.Debug("callback", "task-id", resbody.TaskId, "result", resbody.TaskIsSucceed, "uid", resbody.TaskUid)

		res(resbody)

		return c.SendStatus(200)
	})

	task := model.TaskType{}

	if err := json.Unmarshal([]byte(taskJsonStr), &task); err != nil {
		panic(fmt.Sprintf("json.Unmarshal error, %s", err.Error()))
	}

	replanceQueryTask = &task

	panic(app.Listen(listenIpPort))

}

var replanceQueryTask *model.TaskType

func ApiAndJWT(c *fiber.Ctx) error {

	slog.Debug("ApiAndJWT", "path", c.Route().Path)

	routePath := c.Route().Path
	routePathWithoutStar := strings.TrimSuffix(routePath, "/*")

	path := c.Path()
	pathInDB := strings.TrimPrefix(path, routePathWithoutStar)

	reqHeaders := c.GetReqHeaders()

	uid := reqHeaders["X-Consumer-Custom-Id"]

	if uid == nil {
		return c.SendString(fmt.Sprintf("uid can not be nil"))
	}

	if len(uid) == 0 {
		return c.SendString(fmt.Sprintf("uid can not be 0"))
	}

	uidAsInt, err := strconv.Atoi(uid[0])

	slog.Info("query db param", "pathInDB", pathInDB, "uid", int64(uidAsInt))

	var task *model.TaskType

	if withBillDb {
		var err error
		task, err = cache.Query(pathInDB, int64(uidAsInt))

		if err != nil {
			return c.SendString(fmt.Sprintf("cache.Query %v", err.Error()))
		}
	} else {
		task = replanceQueryTask

	}

	//var cmd string
	//cmd = task.Cmd

	pbMsg := pbUpstream.TaskContent{
		TaskId:          reqHeaders["Task-Id"][0],
		TaskType:        uint64(task.ID),
		TaskKind:        1,
		TaskCmd:         task.Cmd,
		TaskParam:       c.Body(), //[]byte(reqHeaders["Task-Id"][0]),
		TaskTimestamp:   uint64(time.Now().UnixNano()),
		TaskCallback:    "http://" + callbackAddr + "/callback/v1", //"http://192.168.1.10:6001/callback/v1",
		TaskUid:         reqHeaders["X-Consumer-Custom-Id"][0],
		TaskFee:         fmt.Sprintf("%d", task.Price),
		TaskInLen:       int32(len(c.Body())),
		TaskWorkload:    task.Workload,
		ContainerPubkey: []byte(task.PublicKey),
	}

	msgAsJson, err := json.Marshal(pbMsg)

	if err != nil {
		slog.Error("pbUpstream.TaskContent", "Marshal", err.Error())
		return c.SendString(fmt.Sprintf("json.Marshal %v", err.Error()))
	}

	slog.Debug("msgAsJson to kafka", "asjson", string(msgAsJson))

	pbBytes, err := gogoPbProto.Marshal(&pbMsg)

	if err != nil {
		slog.Error("pbUpstream.TaskContent", "gogoPbProto.Marshal", err.Error())
		return c.SendString(fmt.Sprintf("pb error: %v", err.Error()))
	}

	producerMessagesBytes <- bytesAndHeader{
		Bytes: pbBytes,
	}

	//fmt.Println("pbMsg.TaskUid--------------", pbMsg.TaskId)

	asyncMode := false

	// for k, v := range reqHeaders {
	// 	fmt.Println("k-------", k, "v", v)
	// }

	if headerSync, ok := reqHeaders["Async"]; ok {
		//fmt.Println("sync-----------------sync", headerSync)
		for _, syncAsString := range headerSync {
			if syncAsString == "true" {
				asyncMode = true
				break
			}
		}
	}

	slog.Info("task info", "taskid", pbMsg.TaskId, "asyncMode", asyncMode)

	//fmt.Println("asyncMode-----------", asyncMode)

	if asyncMode {
		time.Sleep(10 * time.Second)
		return c.SendString(pbMsg.TaskId)
	} else {
		return syncModeF(c, pbMsg.TaskId)

	}

	//fmt.Println("syncMode-------------", syncMode)
	// resAsJson := ResponseJson{
	// 	TaskId:   resAsPb.TaskId,
	// 	TaskResult: resAsPb.GetTaskResultBody(),
	// 	TaskUid:    resAsPb.TaskUid,
	// 	TaskFee:    resAsPb.TaskFee,
	// 	IsSuccess:  resAsPb.TaskIsSucceed,
	// 	TaskError:  resAsPb.TaskError,
	// }

	//return c.Send(resAsPb.GetTaskResultBody())

	//return c.JSON(resAsJson)

}

func syncModeF(c *fiber.Ctx, taskid string) error {

	wait := req(taskid)
	resAsPb := <-wait

	slog.Debug("resAsPb.TaskResultHeader", "resAsPb.TaskResultHeader", resAsPb.TaskResultHeader)

	//fmt.Println("resAsPb.TaskResultHeader", string(resAsPb.TaskResultHeader), len(resAsPb.TaskResultHeader))

	if resAsPb.TaskResultHeader != nil {
		if len(resAsPb.TaskResultHeader) != 0 {

			headers := make(map[string][]string)

			if err := json.Unmarshal(resAsPb.TaskResultHeader, &headers); err != nil {
				slog.Error("syncModeF", "json.Unmarshal(resAsPb.TaskResultHeader", err.Error())
				return c.SendString(fmt.Sprintf("json.Unmarshal(resAsPb.TaskResultHeader error: %v", err.Error()))
			}

			for k, vs := range headers {
				for _, v := range vs {

					if k == "Content-Encoding" {
						c.Response().Header.SetContentEncoding(v)
					}

					if k == "Content-Type" {
						c.Response().Header.SetContentType(v)
					}
				}
			}
		}
	}
	return c.Send(resAsPb.GetTaskResultBody())

}

/////////////////////////////
//
//
////////////////////////////

// func kafkaProducer() {

// 	// Create a new Sarama configuration for the Kafka producer.
// 	config := sarama.NewConfig()

// 	// Create a new Kafka producer using the specified configuration and broker addresses.
// 	producer, err := sarama.NewAsyncProducer(kafkaBrokers, config)
// 	if err != nil {
// 		panic(fmt.Sprint("Failed to start Kafka producer:", err))
// 	}

// 	// Ensure the Kafka producer is closed when the function ends (deferred execution).
// 	defer producer.Close()

// 	for message := range producerMessages {
// 		counterStr := fmt.Sprintf("%d", counter)

// 		// Get the Indian Standard Time (IST) location
// 		istLocation, err := time.LoadLocation("Asia/Kolkata")
// 		if err != nil {
// 			slog.Error("message := range producerMessages", "Failed to load IST location:", err)
// 			continue
// 		}

// 		// Convert current time to IST
// 		istTime := time.Now().In(istLocation).Format("02-01-2006 15:04:05")
// 		value := fmt.Sprintf("(%s, %s, %s)", counterStr, istTime, message)
// 		producer.Input() <- &sarama.ProducerMessage{
// 			Topic: topic,
// 			Value: sarama.StringEncoder(value),
// 		}

// 		slog.Debug("send to kafka", "msg", string(value))
// 		counter++

// 		// Introduce random delay between 1 to 3 seconds for message push
// 		//time.Sleep(time.Duration(1+rand.Intn(3)) * time.Second)
// 	}
// }

// func kafkaConsumer(wg *sync.WaitGroup) {

// 	// Create a new Sarama configuration for the Kafka producer.
// 	config := sarama.NewConfig()

// 	// Create a new Kafka consumer using the specified configuration and broker addresses.
// 	consumer, err := sarama.NewConsumer(kafkaBrokers, config)
// 	if err != nil {
// 		log.Fatal("Failed to start Kafka consumer:", err)
// 	}

// 	// Ensure the Kafka consumer is closed when the function ends (deferred execution).
// 	defer consumer.Close()

// 	// Create a partition consumer for the specified topic, partition, and starting offset.
// 	// The starting offset is set to sarama.OffsetNewest, which means the consumer will start consuming messages from the latest available offset.
// 	partitionConsumer, err := consumer.ConsumePartition(topic, 0, sarama.OffsetNewest)
// 	if err != nil {
// 		log.Fatal("Failed to start partition consumer:", err)
// 	}

// 	// Ensure the partition consumer is closed when the function ends (deferred execution).
// 	defer partitionConsumer.Close()

// 	// Signal that the consumer goroutine is ready
// 	defer wg.Done()

// 	// Infinite loop to continuously listen for messages from the partitionConsumer.Messages() channel.
// 	for {
// 		select {
// 		case message := <-partitionConsumer.Messages():
// 			value := string(message.Value)
// 			fmt.Printf("Received message from Kafka: %s\n", value)
// 			// Acquire the mutex before appending to the messages slice to ensure concurrent-safe access.
// 			mutex.Lock()
// 			// Append the received message to the internal messages slice.
// 			messages = append(messages, value)
// 			// Release the mutex.
// 			mutex.Unlock()
// 			// Send the received message to the /consumer endpoint via the consumerMessages channel.
// 			consumerMessages <- value
// 		}
// 	}
// }
